@@ -1,12 +1,10 @@
 import numpy as np
 import pygame
 import pymunk
-import pyperclip
 from ev3sim.simulation.interactor import IInteractor
 from ev3sim.simulation.world import World
-from ev3sim.objects.base import objectFactory
 from ev3sim.visual.utils import screenspace_to_worldspace
-from ev3sim.objects.base import STATIC_CATEGORY
+from ev3sim.objects.base import DYNAMIC_CATEGORY
 
 
 class PickUpInteractor(IInteractor):
@@ -35,7 +33,7 @@ class PickUpInteractor(IInteractor):
             if not getattr(self.obj, "clickable", True):
                 self.obj_grabbed = False
             else:
-                self.obj.body.position = self.obj_rel_pos + self.obj_m_pos
+                self.obj.body.position = [a + b for a, b in zip(self.obj_rel_pos, self.obj_m_pos)]
                 idx = (self.position_index + self.position_length) % self.TOTAL_POSITIONS
                 self.positions[(self.position_index + self.position_length) % self.TOTAL_POSITIONS] = self.obj_m_pos
                 self.position_length = min(self.position_length + 1, 10)
@@ -45,29 +43,17 @@ class PickUpInteractor(IInteractor):
         if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
             m_pos = screenspace_to_worldspace(event.pos)
             shapes = World.instance.space.point_query(
-                m_pos, 0.0, pymunk.ShapeFilter(mask=pymunk.ShapeFilter.ALL_MASKS ^ STATIC_CATEGORY)
+                [float(v) for v in m_pos], 0.0, pymunk.ShapeFilter(mask=DYNAMIC_CATEGORY)
             )
             if shapes:
                 max_z = max(pq.shape.obj.clickZ for pq in shapes)
                 shapes = [pq for pq in shapes if pq.shape.obj.clickZ == max_z]
                 self.obj = shapes[0].shape.obj
                 if getattr(self.obj, "clickable", True):
-                    self.obj.body.velocity = np.array([0.0, 0.0])
+                    self.obj.body.velocity = [0.0, 0.0]
                     self.obj_grabbed = True
                     self.obj_rel_pos = self.obj.position - m_pos
                     self.obj_m_pos = m_pos
-        if event.type == pygame.MOUSEBUTTONDOWN and event.button == 3:
-            # If a robot is right clicked, copy it's ID for use in the attach script.
-            m_pos = screenspace_to_worldspace(event.pos)
-            shapes = World.instance.space.point_query(
-                m_pos, 0.0, pymunk.ShapeFilter(mask=pymunk.ShapeFilter.ALL_MASKS ^ STATIC_CATEGORY)
-            )
-            if shapes:
-                max_z = max(pq.shape.obj.clickZ for pq in shapes)
-                shapes = [pq for pq in shapes if pq.shape.obj.clickZ == max_z]
-                self.obj = shapes[0].shape.obj
-                if hasattr(self.obj, "robot_class"):
-                    pyperclip.copy(self.obj.robot_class.ID)
         if event.type == pygame.MOUSEBUTTONUP and event.button == 1 and self.obj_grabbed:
             self.obj_grabbed = False
             # Give velocity based on previous mouse positions.
@@ -84,6 +70,6 @@ class PickUpInteractor(IInteractor):
                 # Sum will return 0 if position length is 0 - we need to handle this.
                 if isinstance(differences, int):
                     differences = np.array([0, 0])
-                self.obj.body.velocity = self.VELOCITY_MULT * differences
+                self.obj.body.velocity = [self.VELOCITY_MULT * d for d in differences]
         if event.type == pygame.MOUSEMOTION and self.obj_grabbed:
             self.obj_m_pos = screenspace_to_worldspace(event.pos)
