@@ -3,7 +3,7 @@ import pygame
 import numpy as np
 import math
 import pymunk
-from ev3sim.events import GAME_RESET, GOAL_SCORED, START_PENALTY, END_PENALTY
+from ev3sim.events import GOAL_SCORED, START_PENALTY, END_PENALTY
 from ev3sim.simulation.interactor import IInteractor
 from ev3sim.simulation.loader import ScriptLoader
 from ev3sim.simulation.randomisation import Randomiser
@@ -13,6 +13,7 @@ from ev3sim.objects.utils import magnitude_sq
 from ev3sim.visual.manager import ScreenObjectManager
 from ev3sim.visual.utils import screenspace_to_worldspace
 from ev3sim.objects.base import DYNAMIC_CATEGORY, STATIC_CATEGORY
+from ev3sim.settings import ObjectSetting
 
 
 class SoccerInteractor(IInteractor):
@@ -30,7 +31,8 @@ class SoccerInteractor(IInteractor):
     BOT_OUT_ON_WHITE_PENALTY_SECONDS = 30
     BALL_RESET_WHITE_DELAY_SECONDS = 5
 
-    TEAM_NAMES = []
+    TEAM_NAME_1 = ""
+    TEAM_NAME_2 = ""
     SPAWN_LOCATIONS = []
     PENALTY_LOCATIONS = []
     GOALS = []
@@ -46,49 +48,34 @@ class SoccerInteractor(IInteractor):
     BOT_SPAWN_RADIUS = 3
     BALL_SPAWN_RADIUS = 1
 
+    # Don't autostart in the `startUp` parent call.
+    AUTOSTART_BOTS = False
+
     _pressed = False
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.START_TIME = datetime.timedelta(minutes=self.GAME_HALF_LENGTH_MINUTES)
-        self.names = self.TEAM_NAMES[:]
-        self.spawns = self.SPAWN_LOCATIONS[:]
-        self.penalty = self.PENALTY_LOCATIONS[:]
-        self.goals = self.GOALS[:]
         self.current_goal_score_tick = -1
         self.out_on_white_tick = 0
         self.time_tick = 0
         self.update_time_text = True
 
-    def locateBots(self):
-        self.robots = []
-        bot_index = 0
-        while True:
-            # Find the next robot.
-            possible_keys = []
-            for key in ScriptLoader.instance.object_map.keys():
-                if key.startswith(f"Robot-{bot_index}"):
-                    possible_keys.append(key)
-            if len(possible_keys) == 0:
-                break
-            possible_keys.sort(key=len)
-            self.robots.append(ScriptLoader.instance.object_map[possible_keys[0]])
-            self.robots[-1].shape.collision_type = self.BOT_COLLISION_TYPE
-            self.robots[-1].shape.bot_index = bot_index
-            bot_index += 1
-        self.bot_penalties = [0] * bot_index
-
-        if len(self.robots) == 0:
-            raise ValueError("No robots loaded.")
-
     def startUp(self):
+        super().startUp()
+        self.START_TIME = datetime.timedelta(minutes=self.GAME_HALF_LENGTH_MINUTES)
+        self.names = [self.TEAM_NAME_1, self.TEAM_NAME_2]
+        self.spawns = self.SPAWN_LOCATIONS[:]
+        self.penalty = self.PENALTY_LOCATIONS[:]
+        self.goals = self.GOALS[:]
         assert len(self.names) == len(self.spawns) and len(self.spawns) == len(
             self.goals
         ), "All player related arrays should be of equal size."
         # Initialise the goal colliders.
         self.goal_colliders = []
-        self.team_scores = []
-        self.locateBots()
+        self.bot_penalties = [0] * len(self.robots)
+        for i, bot in enumerate(self.robots):
+            bot.shape.collision_type = self.BOT_COLLISION_TYPE
+            bot.shape.bot_index = i
         self.BOTS_PER_TEAM = math.ceil(len(self.robots) / len(self.names))
 
         self.ball_centre = objectFactory(
@@ -105,7 +92,11 @@ class SoccerInteractor(IInteractor):
 
         handler = World.instance.space.add_collision_handler(self.BALL_COLLISION_TYPE, self.GOAL_COLLISION_TYPE)
 
+        saved_world_no = World.instance.spawn_no
+
         def handle_collide(arbiter, space, data):
+            if World.instance.spawn_no != saved_world_no:
+                return
             a, b = arbiter.shapes
             if hasattr(a, "goal_index"):
                 self.goalScoredIn(a.goal_index)
@@ -126,11 +117,17 @@ class SoccerInteractor(IInteractor):
                 self.FIELD_BALL_COLLISION_TYPE, self.BALL_COLLISION_TYPE
             )
 
+            saved_world_no = World.instance.spawn_no
+
             def handle_separate_ball(arbiter, space, data):
+                if World.instance.spawn_no != saved_world_no:
+                    return
                 self.out_on_white_tick = self.BALL_RESET_WHITE_DELAY_SECONDS * ScriptLoader.instance.GAME_TICK_RATE
                 return False
 
             def handle_collide_ball(arbiter, space, data):
+                if World.instance.spawn_no != saved_world_no:
+                    return
                 self.out_on_white_tick = 0
                 return False
 
@@ -144,7 +141,11 @@ class SoccerInteractor(IInteractor):
         if self.ENFORCE_OUT_ON_WHITE:
             handler = World.instance.space.add_collision_handler(self.FIELD_COLLISION_TYPE, self.BOT_COLLISION_TYPE)
 
+            saved_world_no = World.instance.spawn_no
+
             def handle_separate(arbiter, space, data):
+                if World.instance.spawn_no != saved_world_no:
+                    return
                 a, b = arbiter.shapes
                 if hasattr(a, "bot_index"):
                     self.penaliseBot(a.bot_index)
@@ -158,6 +159,7 @@ class SoccerInteractor(IInteractor):
 
         for x in range(len(self.names)):
             # Set up goal collider.
+            self.goals[x]["zPos"] = 6
             pos = self.goals[x]["position"]
             del self.goals[x]["position"]
             obj = {
@@ -177,50 +179,48 @@ class SoccerInteractor(IInteractor):
                 ScreenObjectManager.instance.registerVisual(
                     self.goal_colliders[-1].visual, f"Soccer_DEBUG_collider-{len(self.goal_colliders)}"
                 )
-            # Set up team scores
-            self.team_scores.append(0)
             # Set up team name
             ScriptLoader.instance.object_map[f"name{x+1}Text"].text = self.names[x]
-        self.updateScoreText()
-        self.resetPositions()
         for robot in self.robots:
             robot.robot_class.onSpawn()
+        self.reset()
 
     def updateScoreText(self):
         for x in range(len(self.names)):
             ScriptLoader.instance.object_map[f"score{x+1}Text"].text = str(self.team_scores[x])
 
     def reset(self):
-        self.team_scores = [0 for x in self.names]
+        self.team_scores = [0 for _ in self.names]
         self.updateScoreText()
         self.resetPositions()
         self.time_tick = 0
         self.update_time_text = True
-        for robotID in ScriptLoader.instance.robots.keys():
-            ScriptLoader.instance.sendEvent(robotID, GAME_RESET, {})
+        self.restartBots()
 
     def resetPositions(self):
         for team in range(len(self.names)):
             for index in range(self.BOTS_PER_TEAM):
-                actual_index = team * self.BOTS_PER_TEAM + index
+                actual_index = team + index * len(self.names)
                 if actual_index >= len(self.robots):
                     break
                 # Generate a uniformly random point in radius of spawn for bot.
                 diff_radius = Randomiser.random() * self.BOT_SPAWN_RADIUS
                 diff_angle = Randomiser.random() * 2 * np.pi
-                self.robots[actual_index].body.position = self.spawns[team][index][0] + diff_radius * np.array(
-                    [np.cos(diff_angle), np.sin(diff_angle)]
-                )
+                self.robots[actual_index].body.position = [
+                    self.spawns[team][index][0][0] + diff_radius * np.cos(diff_angle),
+                    self.spawns[team][index][0][1] + diff_radius * np.sin(diff_angle),
+                ]
                 self.robots[actual_index].body.angle = self.spawns[team][index][1] * np.pi / 180
-                self.robots[actual_index].body.velocity = np.array([0.0, 0.0])
+                self.robots[actual_index].body.velocity = (0.0, 0.0)
                 self.robots[actual_index].body.angular_velocity = 0
         # Generate position for ball.
         diff_radius = Randomiser.random() * self.BALL_SPAWN_RADIUS
         diff_angle = Randomiser.random() * 2 * np.pi
-        ScriptLoader.instance.object_map["IR_BALL"].body.position = np.array([0, -18]) + diff_radius * np.array(
-            [np.cos(diff_angle), np.sin(diff_angle)]
-        )
-        ScriptLoader.instance.object_map["IR_BALL"].body.velocity = np.array([0.0, 0.0])
+        ScriptLoader.instance.object_map["IR_BALL"].body.position = [
+            diff_radius * np.cos(diff_angle),
+            diff_radius * np.sin(diff_angle) - 18,
+        ]
+        ScriptLoader.instance.object_map["IR_BALL"].body.velocity = (0.0, 0.0)
         for idx in range(len(self.bot_penalties)):
             if self.bot_penalties[idx] > 0:
                 self.finishPenalty(idx)
@@ -231,23 +231,32 @@ class SoccerInteractor(IInteractor):
             key
             for key in all_keys
             if not World.instance.space.point_query(
-                ScriptLoader.instance.object_map[key].position, 0.0, pymunk.ShapeFilter(mask=DYNAMIC_CATEGORY)
+                [float(v) for v in ScriptLoader.instance.object_map[key].position],
+                0.0,
+                pymunk.ShapeFilter(mask=DYNAMIC_CATEGORY),
             )
         ]
         best_key = sorted(
             [
                 (
                     magnitude_sq(
-                        ScriptLoader.instance.object_map["IR_BALL"].body.position
-                        - ScriptLoader.instance.object_map[key].position
+                        [
+                            a - b
+                            for a, b in zip(
+                                ScriptLoader.instance.object_map["IR_BALL"].body.position,
+                                ScriptLoader.instance.object_map[key].position,
+                            )
+                        ]
                     ),
                     key,
                 )
                 for key in (available_keys if available_keys else all_keys)
             ]
         )[0][1]
-        ScriptLoader.instance.object_map["IR_BALL"].body.position = ScriptLoader.instance.object_map[best_key].position
-        ScriptLoader.instance.object_map["IR_BALL"].body.velocity = np.array([0.0, 0.0])
+        ScriptLoader.instance.object_map["IR_BALL"].body.position = [
+            float(v) for v in ScriptLoader.instance.object_map[best_key].position
+        ]
+        ScriptLoader.instance.object_map["IR_BALL"].body.velocity = (0.0, 0.0)
 
     def tick(self, tick):
         super().tick(tick)
@@ -263,23 +272,21 @@ class SoccerInteractor(IInteractor):
 
         # UI Tick
         if self._pressed:
-            ScriptLoader.instance.object_map["controlsReset"].visual.image_path = "assets/ui/controls_reset_pressed.png"
+            ScriptLoader.instance.object_map["controlsReset"].visual.image_path = "ui/controls_reset_pressed.png"
         else:
-            ScriptLoader.instance.object_map[
-                "controlsReset"
-            ].visual.image_path = "assets/ui/controls_reset_released.png"
+            ScriptLoader.instance.object_map["controlsReset"].visual.image_path = "ui/controls_reset_released.png"
         self.update_time()
 
     def afterPhysics(self):
         for team in range(len(self.names)):
             for index in range(self.BOTS_PER_TEAM):
-                actual_index = team * self.BOTS_PER_TEAM + index
+                actual_index = team + index * len(self.names)
                 if actual_index >= len(self.robots):
                     break
                 if self.bot_penalties[actual_index] > 0:
                     self.robots[actual_index].body.position = self.penalty[team][index][0]
                     self.robots[actual_index].body.angle = self.penalty[team][index][1] * np.pi / 180
-                    self.robots[actual_index].position = self.robots[actual_index].body.position
+                    self.robots[actual_index].position = np.array(self.robots[actual_index].body.position)
                     self.robots[actual_index].rotation = self.robots[actual_index].body.angle
 
     @stop_on_pause
@@ -321,7 +328,7 @@ class SoccerInteractor(IInteractor):
         self.current_goal_score_tick = self.cur_tick
         for team in range(len(self.names)):
             for index in range(self.BOTS_PER_TEAM):
-                actual_index = team * self.BOTS_PER_TEAM + index
+                actual_index = team + index * len(self.names)
                 if actual_index >= len(self.robots):
                     break
                 ScriptLoader.instance.sendEvent(
@@ -332,16 +339,17 @@ class SoccerInteractor(IInteractor):
         self.bot_penalties[botIndex] = self.BOT_OUT_ON_WHITE_PENALTY_SECONDS * ScriptLoader.instance.GAME_TICK_RATE
         self.robots[botIndex].clickable = False
         ScriptLoader.instance.sendEvent(f"Robot-{botIndex}", START_PENALTY, {})
-        graphic = self.generatePenaltyGraphic(botIndex)
+        self.generatePenaltyGraphic(botIndex)
 
     def generatePenaltyGraphic(self, botIndex):
-        team = botIndex // self.BOTS_PER_TEAM
+        team = botIndex % len(self.names)
         penaltyIndex = (
             len(
                 [
-                    x
-                    for x in range(team * self.BOTS_PER_TEAM, (team + 1) * self.BOTS_PER_TEAM)
-                    if x < len(self.bot_penalties) and self.bot_penalties[x] != 0
+                    team + x * len(self.names)
+                    for x in range(self.BOTS_PER_TEAM)
+                    if team + x * len(self.names) < len(self.bot_penalties)
+                    and self.bot_penalties[team + x * len(self.names)] != 0
                 ]
             )
             - 1
@@ -386,13 +394,11 @@ class SoccerInteractor(IInteractor):
     def finishPenalty(self, botIndex):
         self.bot_penalties[botIndex] = 0
         self.robots[botIndex].clickable = True
-        self.robots[botIndex].body.position = self.spawns[botIndex // self.BOTS_PER_TEAM][
-            botIndex % self.BOTS_PER_TEAM
-        ][0]
+        self.robots[botIndex].body.position = self.spawns[botIndex % len(self.names)][botIndex // len(self.names)][0]
         self.robots[botIndex].body.angle = (
-            self.spawns[botIndex // self.BOTS_PER_TEAM][botIndex % self.BOTS_PER_TEAM][1] * np.pi / 180
+            self.spawns[botIndex // len(self.names)][botIndex % len(self.names)][1] * np.pi / 180
         )
-        self.robots[botIndex].body.velocity = np.array([0.0, 0.0])
+        self.robots[botIndex].body.velocity = (0.0, 0.0)
         self.robots[botIndex].body.angular_velocity = 0
         ScriptLoader.instance.sendEvent(f"Robot-{botIndex}", END_PENALTY, {})
         ScreenObjectManager.instance.unregisterVisual(f"UI-penalty-{botIndex}")
@@ -404,7 +410,9 @@ class SoccerInteractor(IInteractor):
     def handleEvent(self, event):
         if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
             m_pos = screenspace_to_worldspace(event.pos)
-            shapes = World.instance.space.point_query(m_pos, 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY))
+            shapes = World.instance.space.point_query(
+                [float(v) for v in m_pos], 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY)
+            )
             for shape in shapes:
                 for team in range(len(self.names)):
                     if shape.shape.obj.key.startswith(f"score{team+1}"):
@@ -418,32 +426,94 @@ class SoccerInteractor(IInteractor):
                             raise ValueError(f"Unknown team action {action}")
                         self.updateScoreText()
                     for index in range(self.BOTS_PER_TEAM):
-                        if team * self.BOTS_PER_TEAM + index < len(self.robots):
-                            if shape.shape.obj.key == f"UI-penalty-{team * self.BOTS_PER_TEAM + index}":
-                                self.finishPenalty(team * self.BOTS_PER_TEAM + index)
+                        actual_index = team + index * len(self.names)
+                        if actual_index < len(self.robots):
+                            if shape.shape.obj.key == f"UI-penalty-{actual_index}":
+                                self.finishPenalty(actual_index)
                 if shape.shape.obj.key == "controlsReset":
                     self._pressed = True
 
         if event.type == pygame.MOUSEMOTION:
             m_pos = screenspace_to_worldspace(event.pos)
-            shapes = World.instance.space.point_query(m_pos, 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY))
+            shapes = World.instance.space.point_query(
+                [float(v) for v in m_pos], 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY)
+            )
             for team in range(len(self.names)):
                 for index in range(self.BOTS_PER_TEAM):
+                    actual_index = team + index * len(self.names)
                     for shape in shapes:
-                        if shape.shape.obj.key == f"UI-penalty-{team * self.BOTS_PER_TEAM + index}":
+                        if shape.shape.obj.key == f"UI-penalty-{team + index * len(self.names)}":
                             shape.shape.obj.visual.fill = "penalty_ui_bg_hover"
                             break
                     else:
-                        key = f"UI-penalty-{team * self.BOTS_PER_TEAM + index}"
+                        key = f"UI-penalty-{team + index * len(self.names)}"
                         if key in ScriptLoader.instance.object_map:
                             ScriptLoader.instance.object_map[key].visual.fill = "penalty_ui_bg"
 
         if event.type == pygame.MOUSEBUTTONUP and event.button == 1:
             m_pos = screenspace_to_worldspace(event.pos)
-            shapes = World.instance.space.point_query(m_pos, 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY))
+            shapes = World.instance.space.point_query(
+                [float(v) for v in m_pos], 0.0, pymunk.ShapeFilter(mask=STATIC_CATEGORY)
+            )
             for shape in shapes:
                 if (shape.shape.obj.key == "controlsReset") & self._pressed:
                     self.reset()
                 self._pressed = False
             if len(shapes) == 0:
                 self._pressed = False
+
+
+soccer_settings = {
+    attr: ObjectSetting(SoccerInteractor, attr)
+    for attr in [
+        "TEAM_NAME_1",
+        "TEAM_NAME_2",
+        "SPAWN_LOCATIONS",
+        "PENALTY_LOCATIONS",
+        "GOALS",
+        "GAME_HALF_LENGTH_MINUTES",
+        "SHOW_GOAL_COLLIDERS",
+        "ENFORCE_OUT_ON_WHITE",
+        "BALL_RESET_ON_WHITE",
+        "BALL_RESET_WHITE_DELAY_SECONDS",
+        "BOT_OUT_ON_WHITE_PENALTY_SECONDS",
+    ]
+}
+
+from ev3sim.visual.settings.elements import NumberEntry, TextEntry, Checkbox
+
+visual_settings = [
+    {"height": lambda s: 90, "objects": [TextEntry("__filename__", "BATCH NAME", None, (lambda s: (0, 20)))]},
+    {
+        "height": lambda s: 190,
+        "objects": [
+            TextEntry(["settings", "soccer", "TEAM_NAME_1"], "Team 1", "Team 1 Name", (lambda s: (0, 20))),
+            TextEntry(["settings", "soccer", "TEAM_NAME_2"], "Team 2", "Team 2 Name", (lambda s: (0, 70))),
+            NumberEntry(["settings", "soccer", "GAME_HALF_LENGTH_MINUTES"], 5, "Halftime (m)", (lambda s: (0, 120))),
+        ],
+    },
+    {
+        "height": lambda s: 240 if s[0] < 580 else 140,
+        "objects": [
+            Checkbox(["settings", "soccer", "ENFORCE_OUT_ON_WHITE"], True, "Out on white", (lambda s: (0, 20))),
+            Checkbox(
+                ["settings", "soccer", "BALL_RESET_ON_WHITE"],
+                True,
+                "Ball reset on white",
+                (lambda s: (0, 70) if s[0] < 540 else (s[0] / 2, 20)),
+            ),
+            NumberEntry(
+                ["settings", "soccer", "BOT_OUT_ON_WHITE_PENALTY_SECONDS"],
+                30,
+                "Bot out penalty",
+                (lambda s: (0, 120) if s[0] < 540 else (0, 70)),
+            ),
+            NumberEntry(
+                ["settings", "soccer", "BALL_RESET_WHITE_DELAY_SECONDS"],
+                5,
+                "Ball reset delay",
+                (lambda s: (0, 170) if s[0] < 540 else (s[0] / 2, 70)),
+            ),
+        ],
+    },
+]
