@@ -7,6 +7,7 @@ from typing import Optional, Tuple
 from ev3sim.visual.manager import ScreenObjectManager
 import ev3sim.visual.utils as utils
 from ev3sim.objects.utils import local_space_to_world_space
+from ev3sim.search_locations import asset_locations
 
 USE_PYGAME_GFX = True
 
@@ -23,6 +24,8 @@ class IVisualElement:
     sensorVisible: bool
     """Specifies whether the visual object should affect the colour sensor readings."""
 
+    customMap = None
+
     def __init__(self, **kwargs):
         self.initFromKwargs(**kwargs)
 
@@ -34,6 +37,12 @@ class IVisualElement:
         self.sensorVisible = kwargs.get("sensorVisible", False)
         self.visible = kwargs.get("visible", True)
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.position = (
+            pos[0] + amount * (self.position[0] - pos[0]),
+            pos[1] + amount * (self.position[1] - pos[1]),
+        )
+
     @property
     def position(self) -> np.ndarray:
         """
@@ -72,7 +81,7 @@ class IVisualElement:
         except AttributeError:
             pass
 
-    def applyToScreen(self):
+    def applyToScreen(self, screen):
         """
         A method that all visual elements must implement.
         When invoked, should apply themselves to the screen.
@@ -117,6 +126,10 @@ class Colorable(IVisualElement):
         self.stroke = kwargs.get("stroke", None)
         self.stroke_width = kwargs.get("stroke_width", 1)
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.stroke_width *= amount
+        super().scaleAtPosition(amount, pos=pos)
+
     @property
     def fill(self) -> Tuple[int]:
         return self._fill
@@ -149,23 +162,46 @@ class Colorable(IVisualElement):
 
     @property
     def scaledStrokeWidth(self):
+        if self.customMap is None:
+            return max(
+                1,
+                int(
+                    self.stroke_width
+                    * ScreenObjectManager.instance.SCREEN_WIDTH
+                    / ScreenObjectManager.instance.MAP_WIDTH
+                ),
+            )
         return max(
             1,
-            int(self.stroke_width * ScreenObjectManager.instance.SCREEN_WIDTH / ScreenObjectManager.instance.MAP_WIDTH),
+            int(self.stroke_width * self.customMap["SCREEN_WIDTH"] / self.customMap["MAP_WIDTH"]),
         )
 
 
 class Image(Colorable):
     def initFromKwargs(self, **kwargs):
         self._image_path = ""
-        self.image_path = kwargs.get("image_path")
         super().initFromKwargs(**kwargs)
         self.fill = kwargs.get("fill", (0, 0, 0, 0))
         self.hAlignment = kwargs.get("hAlignment", "m")
         self.vAlignment = kwargs.get("vAlignment", "m")
         self.scale = kwargs.get("scale", 1)
+        self.flip = kwargs.get("flip", [False, False])
+        self.image_path = kwargs.get("image_path")
         self.calculatePoints()
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        if isinstance(self.scale, (tuple, list)):
+            if isinstance(amount, (tuple, list)):
+                self.scale = (self.scale[0] * amount[0], self.scale[1] * amount[1])
+            else:
+                self.scale = (self.scale[0] * amount, self.scale[1] * amount)
+        else:
+            if isinstance(amount, (tuple, list)):
+                self.scale = (self.scale * amount[0], self.scale * amount[1])
+            else:
+                self.scale *= amount
+        super().scaleAtPosition(amount, pos=pos)
+
     @property
     def image_path(self):
         return self._image_path
@@ -174,19 +210,39 @@ class Image(Colorable):
     def image_path(self, value):
         from ev3sim.file_helper import find_abs
 
-        self._image_path = find_abs(value, allowed_areas=["local", "local/assets/", "package", "package/assets/"])
-        self.image = pygame.image.load(self._image_path)
+        image_path = find_abs(value, allowed_areas=asset_locations())
+        if image_path != self._image_path:
+            self._image_path = image_path
+            self.image = pygame.image.load(self._image_path)
+            try:
+                self.calculatePoints()
+            except:
+                pass
 
     def calculatePoints(self):
-        relative_scale = ScreenObjectManager.instance.relativeScreenScale()
+        if self.customMap is None:
+            relative_scale = ScreenObjectManager.instance.relativeScreenScale()
+            # In order to have a reasonably sized image at all resolutions, calculate the scale to use based on the starting screen scale as well.
+            relative_scale = relative_scale * ScreenObjectManager.instance.original_SCREEN_WIDTH / 1280
+        else:
+            relative_scale = self.customMap["SCREEN_WIDTH"] / 1280 * 293.3 / self.customMap["MAP_WIDTH"]
         new_size = [
-            int(self.image.get_size()[0] * self.scale * relative_scale[0]),
-            int(self.image.get_size()[1] * self.scale * relative_scale[1]),
+            int(
+                self.image.get_size()[0]
+                * (self.scale[0] if isinstance(self.scale, (list, tuple)) else self.scale)
+                * relative_scale
+            ),
+            int(
+                self.image.get_size()[1]
+                * (self.scale[1] if isinstance(self.scale, (list, tuple)) else self.scale)
+                * relative_scale
+            ),
         ]
         scaled = pygame.transform.scale(self.image, new_size)
-        self.rotated = pygame.transform.rotate(scaled, self.rotation * 180 / np.pi)
+        flipped = pygame.transform.flip(scaled, self.flip[0], self.flip[1])
+        self.rotated = pygame.transform.rotate(flipped, self.rotation * 180 / np.pi)
         self.rotated.fill(self.fill, special_flags=pygame.BLEND_ADD)
-        self.screen_location = utils.worldspace_to_screenspace(self.position)
+        self.screen_location = utils.worldspace_to_screenspace(self.position, self.customMap)
         self.screen_size = self.rotated.get_size()
         if self.hAlignment == "l":
             pass
@@ -206,12 +262,22 @@ class Image(Colorable):
             raise ValueError(f"vAlignment is incorrect: {self.vAlignment}")
         from ev3sim.visual.utils import screenspace_to_worldspace
 
-        physics_size = screenspace_to_worldspace(
-            [
-                ScreenObjectManager.instance.SCREEN_WIDTH / 2 + self.screen_size[0],
-                ScreenObjectManager.instance.SCREEN_HEIGHT / 2 + self.screen_size[1],
-            ]
-        )
+        if self.customMap is None:
+            physics_size = screenspace_to_worldspace(
+                [
+                    ScreenObjectManager.instance._SCREEN_WIDTH_ACTUAL / 2 + self.screen_size[0],
+                    ScreenObjectManager.instance._SCREEN_HEIGHT_ACTUAL / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
+        else:
+            physics_size = screenspace_to_worldspace(
+                [
+                    self.customMap["SCREEN_WIDTH"] / 2 + self.screen_size[0],
+                    self.customMap["SCREEN_HEIGHT"] / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
         self.verts = [
             (physics_size[0] / 2, physics_size[1] / 2),
             (physics_size[0] / 2, -physics_size[1] / 2),
@@ -219,8 +285,8 @@ class Image(Colorable):
             (-physics_size[0] / 2, physics_size[1] / 2),
         ]
 
-    def applyToScreen(self):
-        ScreenObjectManager.instance.screen.blit(self.rotated, self.screen_location)
+    def applyToScreen(self, screen):
+        screen.blit(self.rotated, self.screen_location)
 
     def generateBodyAndShape(self, physObj, body=None, rel_pos=(0, 0)):
         if body is None:
@@ -253,12 +319,22 @@ class Image(Colorable):
         res = np.array([0.0, 0.0])
         from ev3sim.visual.utils import screenspace_to_worldspace
 
-        physics_size = screenspace_to_worldspace(
-            [
-                ScreenObjectManager.instance.SCREEN_WIDTH / 2 + self.screen_size[0],
-                ScreenObjectManager.instance.SCREEN_HEIGHT / 2 + self.screen_size[1],
-            ]
-        )
+        if self.customMap is None:
+            physics_size = screenspace_to_worldspace(
+                [
+                    ScreenObjectManager.instance._SCREEN_WIDTH_ACTUAL / 2 + self.screen_size[0],
+                    ScreenObjectManager.instance._SCREEN_HEIGHT_ACTUAL / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
+        else:
+            physics_size = screenspace_to_worldspace(
+                [
+                    self.customMap["SCREEN_WIDTH"] / 2 + self.screen_size[0],
+                    self.customMap["SCREEN_HEIGHT"] / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
         if self.hAlignment == "l":
             res += np.array([physics_size[0] / 2, 0.0])
         elif self.hAlignment == "m":
@@ -287,52 +363,63 @@ class Line(Colorable):
         self.end = kwargs.get("end")
         super().initFromKwargs(**kwargs)
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.start = (
+            pos[0] + amount * (self.start[0] - pos[0]),
+            pos[1] + amount * (self.start[1] - pos[1]),
+        )
+        self.end = (
+            pos[0] + amount * (self.end[0] - pos[0]),
+            pos[1] + amount * (self.end[1] - pos[1]),
+        )
+        super().scaleAtPosition(amount, pos=pos)
+
     def calculatePoints(self):
         return
 
-    def _applyToScreen(self):
+    def _applyToScreen(self, screen):
         if self.stroke and self.stroke_width:
             pygame.draw.line(
-                ScreenObjectManager.instance.screen,
+                screen,
                 self.fill,
-                utils.worldspace_to_screenspace(self.start),
-                utils.worldspace_to_screenspace(self.end),
+                utils.worldspace_to_screenspace(self.start, self.customMap),
+                utils.worldspace_to_screenspace(self.end, self.customMap),
                 self.scaledStrokeWidth,
             )
         elif self.fill:
             pygame.draw.line(
-                ScreenObjectManager.instance.screen,
+                screen,
                 self.fill,
-                utils.worldspace_to_screenspace(self.start),
-                utils.worldspace_to_screenspace(self.end),
+                utils.worldspace_to_screenspace(self.start, self.customMap),
+                utils.worldspace_to_screenspace(self.end, self.customMap),
                 1,
             )
 
-    def _applyToScreenGfx(self):
+    def _applyToScreenGfx(self, screen):
         # Note: aaline() isn't actually from gfxdraw,
         # it just seemed nicer to keep all the AA options toggleable together
 
         if self.stroke and self.stroke_width:
             pygame.draw.aaline(
-                ScreenObjectManager.instance.screen,
+                screen,
                 self.fill,
-                utils.worldspace_to_screenspace(self.start),
-                utils.worldspace_to_screenspace(self.end),
+                utils.worldspace_to_screenspace(self.start, self.customMap),
+                utils.worldspace_to_screenspace(self.end, self.customMap),
                 self.scaledStrokeWidth,
             )
         elif self.fill:
             pygame.draw.aaline(
-                ScreenObjectManager.instance.screen,
+                screen,
                 self.fill,
-                utils.worldspace_to_screenspace(self.start),
-                utils.worldspace_to_screenspace(self.end),
+                utils.worldspace_to_screenspace(self.start, self.customMap),
+                utils.worldspace_to_screenspace(self.end, self.customMap),
             )
 
-    def applyToScreen(self):
+    def applyToScreen(self, screen):
         if USE_PYGAME_GFX:
-            self._applyToScreenGfx()
+            self._applyToScreenGfx(screen)
         else:
-            self._applyToScreen()
+            self._applyToScreen(screen)
 
 
 class Polygon(Colorable):
@@ -344,6 +431,16 @@ class Polygon(Colorable):
         self.points = [None] * len(self.verts)
         super().initFromKwargs(**kwargs)
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.verts = [
+            (
+                pos[0] + amount * (v[0] - pos[0]),
+                pos[1] + amount * (v[1] - pos[1]),
+            )
+            for v in self.verts
+        ]
+        super().scaleAtPosition(amount, pos=pos)
+
     def calculatePoints(self):
         try:
             tmp = self.rotation, self.position
@@ -351,36 +448,37 @@ class Polygon(Colorable):
             return
         for i, v in enumerate(self.verts):
             self.points[i] = utils.worldspace_to_screenspace(
-                local_space_to_world_space(v, self.rotation, self.position)
+                local_space_to_world_space(v, self.rotation, self.position),
+                self.customMap,
             )
 
-    def _applyToScreen(self):
+    def _applyToScreen(self, screen):
         if self.fill:
-            pygame.draw.polygon(ScreenObjectManager.instance.screen, self.fill, self.points)
+            pygame.draw.polygon(screen, self.fill, self.points)
         if self.stroke and self.stroke_width:
-            pygame.draw.polygon(ScreenObjectManager.instance.screen, self.stroke, self.points, self.scaledStrokeWidth)
+            pygame.draw.polygon(screen, self.stroke, self.points, self.scaledStrokeWidth)
 
-    def _applyToScreenGfx(self):
+    def _applyToScreenGfx(self, screen):
         import pygame.gfxdraw
 
         if self.fill:
-            pygame.gfxdraw.aapolygon(ScreenObjectManager.instance.screen, self.points, self.fill)
-            pygame.gfxdraw.filled_polygon(ScreenObjectManager.instance.screen, self.points, self.fill)
+            pygame.gfxdraw.aapolygon(screen, self.points, self.fill)
+            pygame.gfxdraw.filled_polygon(screen, self.points, self.fill)
 
         if self.stroke and self.stroke_width:
             stroke_width = self.scaledStrokeWidth
 
             if stroke_width > 1:
-                pygame.draw.polygon(ScreenObjectManager.instance.screen, self.stroke, self.points, stroke_width)
+                pygame.draw.polygon(screen, self.stroke, self.points, stroke_width)
             else:
-                pygame.gfxdraw.aapolygon(ScreenObjectManager.instance.screen, self.points, self.stroke)
-                pygame.gfxdraw.polygon(ScreenObjectManager.instance.screen, self.points, self.stroke)
+                pygame.gfxdraw.aapolygon(screen, self.points, self.stroke)
+                pygame.gfxdraw.polygon(screen, self.points, self.stroke)
 
-    def applyToScreen(self):
+    def applyToScreen(self, screen):
         if USE_PYGAME_GFX:
-            self._applyToScreenGfx()
+            self._applyToScreenGfx(screen)
         else:
-            self._applyToScreen()
+            self._applyToScreen(screen)
 
     def generateBodyAndShape(self, physObj, body=None, rel_pos=(0, 0)):
         if body is None:
@@ -435,65 +533,64 @@ class Circle(Colorable):
         self.radius = kwargs.get("radius", 20)
         super().initFromKwargs(**kwargs)
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.radius *= amount
+        super().scaleAtPosition(amount, pos=pos)
+
     def calculatePoints(self):
         try:
             tmp = self.radius
         except:
             return
-        self.point = utils.worldspace_to_screenspace(self.position)
-        self.v_radius = int(
-            ScreenObjectManager.instance.SCREEN_HEIGHT / ScreenObjectManager.instance.MAP_HEIGHT * self.radius
-        )
-        self.h_radius = int(
-            ScreenObjectManager.instance.SCREEN_WIDTH / ScreenObjectManager.instance.MAP_WIDTH * self.radius
-        )
+        self.point = utils.worldspace_to_screenspace(self.position, self.customMap)
+        if self.customMap is None:
+            self.v_radius = int(
+                ScreenObjectManager.instance.SCREEN_HEIGHT / ScreenObjectManager.instance.MAP_HEIGHT * self.radius
+            )
+            self.h_radius = int(
+                ScreenObjectManager.instance.SCREEN_WIDTH / ScreenObjectManager.instance.MAP_WIDTH * self.radius
+            )
+        else:
+            self.v_radius = int(self.customMap["SCREEN_HEIGHT"] / self.customMap["MAP_HEIGHT"] * self.radius)
+            self.h_radius = int(self.customMap["SCREEN_WIDTH"] / self.customMap["MAP_WIDTH"] * self.radius)
         self.rect = pygame.Rect(
             self.point[0] - self.h_radius, self.point[1] - self.v_radius, self.h_radius * 2, self.v_radius * 2
         )
 
-    def _applyToScreen(self):
+    def _applyToScreen(self, screen):
         if self.fill:
-            pygame.draw.ellipse(ScreenObjectManager.instance.screen, self.fill, self.rect)
+            pygame.draw.ellipse(screen, self.fill, self.rect)
         if self.stroke and self.stroke_width:
-            pygame.draw.ellipse(ScreenObjectManager.instance.screen, self.stroke, self.rect, self.scaledStrokeWidth)
+            pygame.draw.ellipse(screen, self.stroke, self.rect, self.scaledStrokeWidth)
 
-    def _applyToScreenGfx(self):
+    def _applyToScreenGfx(self, screen):
         import pygame.gfxdraw
 
         if self.fill and self.stroke and self.stroke_width:
             stroke_width = self.scaledStrokeWidth
 
-            pygame.gfxdraw.aaellipse(
-                ScreenObjectManager.instance.screen, *self.point, self.h_radius, self.v_radius, self.stroke
-            )
-            pygame.gfxdraw.filled_ellipse(
-                ScreenObjectManager.instance.screen, *self.point, self.h_radius, self.v_radius, self.stroke
-            )
+            pygame.gfxdraw.aaellipse(screen, *self.point, self.h_radius, self.v_radius, self.stroke)
+            pygame.gfxdraw.filled_ellipse(screen, *self.point, self.h_radius, self.v_radius, self.stroke)
 
             # Assumes stroke_width >= radius implies fill with stroke colour
             h_fill_radius = max(int(self.h_radius - stroke_width), 0)
             v_fill_radius = max(int(self.v_radius - stroke_width), 0)
 
             if h_fill_radius and v_fill_radius:
-                pygame.gfxdraw.aaellipse(
-                    ScreenObjectManager.instance.screen, *self.point, h_fill_radius, v_fill_radius, self.fill
-                )
-                pygame.gfxdraw.filled_ellipse(
-                    ScreenObjectManager.instance.screen, *self.point, h_fill_radius, v_fill_radius, self.fill
-                )
+                pygame.gfxdraw.aaellipse(screen, *self.point, h_fill_radius, v_fill_radius, self.fill)
+                pygame.gfxdraw.filled_ellipse(screen, *self.point, h_fill_radius, v_fill_radius, self.fill)
         elif self.fill:
-            pygame.gfxdraw.aaellipse(
-                ScreenObjectManager.instance.screen, *self.point, self.h_radius, self.v_radius, self.fill
-            )
-            pygame.gfxdraw.filled_ellipse(
-                ScreenObjectManager.instance.screen, *self.point, self.h_radius, self.v_radius, self.fill
-            )
+            pygame.gfxdraw.aaellipse(screen, *self.point, self.h_radius, self.v_radius, self.fill)
+            pygame.gfxdraw.filled_ellipse(screen, *self.point, self.h_radius, self.v_radius, self.fill)
+        elif self.stroke and self.stroke_width:
+            # No fill but still stroke and stroke width. Can't use gfxdraw.
+            self._applyToScreen(screen)
 
-    def applyToScreen(self):
+    def applyToScreen(self, screen):
         if USE_PYGAME_GFX:
-            self._applyToScreenGfx()
+            self._applyToScreenGfx(screen)
         else:
-            self._applyToScreen()
+            self._applyToScreen(screen)
 
     def generateBodyAndShape(self, physObj, body=None, rel_pos=(0, 0)):
         if body is None:
@@ -501,7 +598,7 @@ class Circle(Colorable):
             body = pymunk.Body(
                 physObj.mass, moment, body_type=pymunk.Body.STATIC if physObj.static else pymunk.Body.DYNAMIC
             )
-        shape = pymunk.Circle(body, self.radius, offset=rel_pos)
+        shape = pymunk.Circle(body, self.radius, offset=[float(v) for v in rel_pos])
         shape.friction = physObj.friction_coefficient
         shape.elasticity = physObj.restitution_coefficient
         shape.collision_type = 1
@@ -512,6 +609,43 @@ class Circle(Colorable):
         return body, shape
 
 
+class Arc(Polygon):
+    """
+    An Arc is just a circle which only has a pie portion drawn.
+
+    However pygame filled arcs (or thick arcs) aren't drawn well. So just use a polygon!
+    """
+
+    def initFromKwargs(self, **kwargs):
+        self.radius = kwargs.get("radius", 1)
+        self.angle_span = kwargs.get("angle", 90)
+        kwargs["verts"] = [
+            [self.radius * np.cos(x * np.pi / 180), self.radius * np.sin(x * np.pi / 180)]
+            for x in range(int(self.angle_span))
+        ]
+        super().initFromKwargs(**kwargs)
+        if self.fill is not None:
+            raise ValueError("Arcs are only for strokes.")
+        # Actually fill it though.
+        self.stroke, self.fill = self.fill, self.stroke
+        # But actually, we want to double back so we draw the stroke correctly.
+        self.verts = [
+            [
+                (self.radius + self.stroke_width / 2) * np.cos(x * np.pi / 180 * (-1 if self.angle_span < 0 else 1)),
+                (self.radius + self.stroke_width / 2) * np.sin(x * np.pi / 180 * (-1 if self.angle_span < 0 else 1)),
+            ]
+            for x in range(abs(self.angle_span) + 1)
+        ] + [
+            [
+                (self.radius - self.stroke_width / 2) * np.cos(x * np.pi / 180 * (-1 if self.angle_span < 0 else 1)),
+                (self.radius - self.stroke_width / 2) * np.sin(x * np.pi / 180 * (-1 if self.angle_span < 0 else 1)),
+            ]
+            for x in range(abs(self.angle_span), -1, -1)
+        ]
+        self.points = [None] * len(self.verts)
+        self.calculatePoints()
+
+
 class Text(Colorable):
 
     font_style: str
@@ -531,33 +665,41 @@ class Text(Colorable):
         super().initFromKwargs(**kwargs)
         from ev3sim.file_helper import find_abs
 
-        self.font_style = kwargs.get("font_style", "OpenSans-SemiBold.ttf")
-        self.font_path = find_abs(
-            self.font_style, allowed_areas=["local/assets/", "local", "package/assets/", "package"]
-        )
+        self.font_style = kwargs.get("font_style", "fonts/OpenSans-SemiBold.ttf")
+        self.font_path = find_abs(self.font_style, allowed_areas=asset_locations())
         self.font_size = kwargs.get("font_size", 30)
-        self.font = pygame.freetype.Font(self.font_path, self.font_size)
         self.hAlignment = kwargs.get("hAlignment", "l")
         self.vAlignment = kwargs.get("vAlignment", "t")
         self.text = kwargs.get("text", "Test")
 
+    def scaleAtPosition(self, amount, pos=(0, 0)):
+        self.font_size = int(self.font_size * amount)
+        super().scaleAtPosition(amount, pos=pos)
+
     def calculatePoints(self):
-        if not hasattr(self, "font"):
-            return
+        if self.customMap is None:
+            relative_scale = ScreenObjectManager.instance.relativeScreenScale()
+            # In order to have a reasonably sized image at all resolutions, calculate the scale to use based on the starting screen scale as well.
+            relative_scale = relative_scale * ScreenObjectManager.instance.original_SCREEN_WIDTH / 1280
+        else:
+            relative_scale = self.customMap["SCREEN_WIDTH"] / 1280 * 293.3 / self.customMap["MAP_WIDTH"]
+        new_font_size = int(self.font_size * relative_scale)
+        # Scale the font size as much as possible
+        relative_scale = self.font_size * relative_scale / new_font_size
+        self.font = pygame.freetype.Font(self.font_path, new_font_size)
         self.surface, self.rect = self.font.render(self.text, fgcolor=self.fill)
-        relative_scale = ScreenObjectManager.instance.relativeScreenScale()
         self.surface = pygame.transform.scale(
             self.surface,
-            (int(self.surface.get_width() * relative_scale[0]), int(self.surface.get_height() * relative_scale[1])),
+            (int(self.surface.get_width() * relative_scale), int(self.surface.get_height() * relative_scale)),
         )
         self.screen_size = (self.surface.get_width(), self.surface.get_height())
-        baseline = np.array([self.rect.x * relative_scale[0], self.rect.y * relative_scale[1]])
+        baseline = np.array([self.rect.x * relative_scale, self.rect.y * relative_scale])
         self.rect.move_ip(-self.rect.x, -self.rect.y)
         width, height = (
-            self.font.get_rect(self.text).width * relative_scale[0],
-            self.font.get_rect(self.text).height * relative_scale[1],
+            self.font.get_rect(self.text).width * relative_scale,
+            self.font.get_rect(self.text).height * relative_scale,
         )
-        self.anchor = utils.worldspace_to_screenspace(self.position)
+        self.anchor = utils.worldspace_to_screenspace(self.position, self.customMap)
         if self.hAlignment == "l":
             pass
         elif self.hAlignment == "m":
@@ -578,19 +720,29 @@ class Text(Colorable):
             raise ValueError(f"vAlignment is incorrect: {self.vAlignment}")
         self.rect.move_ip(*self.anchor)
 
-    def applyToScreen(self):
-        ScreenObjectManager.instance.screen.blit(self.surface, self.rect)
+    def applyToScreen(self, screen):
+        screen.blit(self.surface, self.rect)
 
     def getPositionAnchorOffset(self):
         res = np.array([0.0, 0.0])
         from ev3sim.visual.utils import screenspace_to_worldspace
 
-        physics_size = screenspace_to_worldspace(
-            [
-                ScreenObjectManager.instance.SCREEN_WIDTH / 2 + self.screen_size[0],
-                ScreenObjectManager.instance.SCREEN_HEIGHT / 2 + self.screen_size[1],
-            ]
-        )
+        if self.customMap is None:
+            physics_size = screenspace_to_worldspace(
+                [
+                    ScreenObjectManager.instance._SCREEN_WIDTH_ACTUAL / 2 + self.screen_size[0],
+                    ScreenObjectManager.instance._SCREEN_HEIGHT_ACTUAL / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
+        else:
+            physics_size = screenspace_to_worldspace(
+                [
+                    self.customMap["SCREEN_WIDTH"] / 2 + self.screen_size[0],
+                    self.customMap["SCREEN_HEIGHT"] / 2 + self.screen_size[1],
+                ],
+                self.customMap,
+            )
         if self.hAlignment == "l":
             res += np.array([physics_size[0] / 2, 0.0])
         elif self.hAlignment == "m":
@@ -613,7 +765,7 @@ class Text(Colorable):
 def visualFactory(**options):
     if "name" not in options:
         raise ValueError("Tried to generate visual element, but no 'name' field was supplied.")
-    for klass in (Polygon, Rectangle, Circle, Text, Image):
+    for klass in (Polygon, Rectangle, Circle, Arc, Text, Image):
         if options["name"] == klass.__name__:
             r = klass(**options)
             return r
